
_itf_embedded = dict()

def Register(key, value):
	"""Register an embedded class"""
	_itf_embedded[key] = value

def Lookup(key):
	"""Lookup an embedded class"""
	return _itf_embedded.get(key, None)



def marshal(output, name, value):
	func = name + '__marshal'
	getattr(output, func)(value)

def demarshal(input, name):
	func = name + '__demarshal'
	return getattr(input, func)()

def check(type, value):
	if isinstance(type, str) :
		if type == 'char' :
			if len(value) != 1 :
				raise SystemException("IDL:CORBA/BAD_PARAM:1.0", 2, CORBA_COMPLETED_MAYBE)
		elif type == 'wchar' :
			if len(value) != 1 :
				raise SystemException("IDL:CORBA/BAD_PARAM:1.0", 2, CORBA_COMPLETED_MAYBE)
		elif type == 'octet' :
			if value < 0 or value > 255 :
				raise SystemException("IDL:CORBA/BAD_PARAM:1.0", 2, CORBA_COMPLETED_MAYBE)
		elif type == 'short' :
			if value < -32768 or value > 32767 :
				raise SystemException("IDL:CORBA/BAD_PARAM:1.0", 2, CORBA_COMPLETED_MAYBE)
		elif type == 'unsigned_short' :
			if value < 0 or value > 65535 :
				raise SystemException("IDL:CORBA/BAD_PARAM:1.0", 2, CORBA_COMPLETED_MAYBE)
		elif type == 'long' :
			if value < -2147483648 or value > 2147483647 :
				raise SystemException("IDL:CORBA/BAD_PARAM:1.0", 2, CORBA_COMPLETED_MAYBE)
		elif type == 'unsigned_long' :
			if value < 0 or value > 4294967295L :
				raise SystemException("IDL:CORBA/BAD_PARAM:1.0", 2, CORBA_COMPLETED_MAYBE)
		elif type == 'long_long' :
			if value < -9223372036854775808L or value > 9223372036854775807L :
				raise SystemException("IDL:CORBA/BAD_PARAM:1.0", 2, CORBA_COMPLETED_MAYBE)
		elif type == 'unsigned_long_long' :
			if value < 0 or value > 18446744073709551615L :
				raise SystemException("IDL:CORBA/BAD_PARAM:1.0", 2, CORBA_COMPLETED_MAYBE)
		elif type == 'float' :
			pass
		elif type == 'double' :
			pass
		elif type == 'long_double' :
			pass
		elif type == 'boolean' :
			pass
		elif type == 'string' :
			pass
		elif type == 'wstring' :
			pass
		else :
			raise "Internal Error : %s" % type
	else :
		if isinstance(value, type) == False :
			raise SystemException("IDL:CORBA/BAD_PARAM:1.0", 2, CORBA_COMPLETED_MAYBE)

class UserException(Exception):
	"""An IDL exception is translated into a Python class derived from CORBA.UserException."""
	pass

class SystemException(Exception):      
	"""CORBA.SystemException"""

	def __init__(self, repos_id, minor, completed):
		self.repos_id = repos_id
		self.minor = minor
		self.completed = completed

	def __str__(self):
		return self.repos_id

CORBA_COMPLETED_YES = 0    # The object implementation has completed
                           # processing prior to the exception being raised.
CORBA_COMPLETED_NO = 1     # The object implementation was never initiated
                           # prior to the exception being raised.
CORBA_COMPLETED_MAYBE = 2  # The status of implementation completion is
                           # indeterminate.

class Enum(object):
	"""base class for IDL enum"""

	def __init__(self, str, val):
		self._val = val
		self._enum_str[val] = str
		self._enum[val] = self

	def marshal(self, output):
		output.long__marshal(self._val)

	def demarshal(cls, input):
		val = input.long__demarshal()
		if cls._enum.has_key(val) :
			return cls._enum[val]
		else :
			raise 'CORBA.MARSHAL'
	demarshal = classmethod(demarshal)

	def __repr__(self):
		return self._enum_str[self._val]





import PyIDL as CORBA
import PyIDL.cdr as CDR
import PyIDL.iop as IOP
import PyIDL.giop as GIOP

_request_id = 0

def _getRequestId():
	global _request_id
	_request_id += 1
	return _request_id

def RequestOneWay(sock, request_header, request_body):
	request_header.request_id = _getRequestId()
	request = CDR.OutputBuffer()
	request_header.marshal(request)
	request.write(request_body.getvalue())
	msg = CDR.OutputBuffer()
	GIOP.MessageHeader_1_1(
			magic="GIOP",
			GIOP_version=GIOP.Version(major=1, minor=2),
			flags=0x01,	# flags : little endian
			message_type=0,		# Request
			message_size=len(request.getvalue())
	).marshal(msg)
	msg.write(request.getvalue())
	request.close()
	sock.send(msg.getvalue())
	msg.close()

def RequestReply(sock, request_header, request_body):
	request_header.request_id = _getRequestId()
	request = CDR.OutputBuffer()
	request_header.marshal(request)
	request.write(request_body.getvalue())
	msg = CDR.OutputBuffer()
	GIOP.MessageHeader_1_1(
			magic="GIOP",
			GIOP_version=GIOP.Version(major=1, minor=2),
			flags=0x01,	# flags : little endian
			message_type=0,		# Request
			message_size=len(request.getvalue())
	).marshal(msg)
	msg.write(request.getvalue())
	request.close()
	sock.send(msg.getvalue())
	msg.close()
	while True :
		_header = sock.recv(12)
		header = CDR.InputBuffer(_header)
		magic = ''
		magic += CORBA.demarshal(header, 'char')
		magic += CORBA.demarshal(header, 'char')
		magic += CORBA.demarshal(header, 'char')
		magic += CORBA.demarshal(header, 'char')
		GIOP_version = GIOP.Version.demarshal(header)
		flags = CORBA.demarshal(header, 'octet')
		endian = flags & 0x01
		header.endian = endian			# now, endian is known
		message_type = CORBA.demarshal(header, 'octet')
		message_size = CORBA.demarshal(header, 'unsigned_long')

		if magic == 'GIOP' and GIOP_version.major == 1 and GIOP_version.minor == 2 and message_type == 1 :
			_reply = ''
			while (message_size > 0) :
				reply_i = sock.recv(message_size)
				_reply += reply_i
				message_size -= len(reply_i)
			reply = CDR.InputBuffer(_reply, endian)
			reply_header = GIOP.ReplyHeader_1_2.demarshal(reply)
			if request_header.request_id == reply_header.request_id :
#				print "reply id %d" % reply_header.request_id
				return (reply_header.reply_status, reply_header.service_context, reply)
			elif request_id > reply_header.request_id :
				print "bad request id %d (waiting %d).\n" % (reply_header.request_id, request_header.request_id)
#				goto RETRY
			else :
				print "bad request id %d (waiting %d).\n" % (reply_header.request_id, request_header.request_id)
				raise CORBA.SystemException("IDL:CORBA/INTERNAL:1.0", 8, CORBA.CORBA_COMPLETED_MAYBE)
		else :
			print "bad header."
			raise CORBA.SystemException("IDL:CORBA/INTERNAL:1.0", 8, CORBA.CORBA_COMPLETED_MAYBE)

class Servant(object):
	def __init__(self):
		self.itf = dict()

	def Register(self, key, value):
		self.itf[key] = value

	def Servant(self, request):
		message = CDR.InputBuffer(request)
		magic = ''
		magic += CORBA.demarshal(message, 'char')
		magic += CORBA.demarshal(message, 'char')
		magic += CORBA.demarshal(message, 'char')
		magic += CORBA.demarshal(message, 'char')
		GIOP_version = GIOP.Version.demarshal(message)
		flags = CORBA.demarshal(message, 'octet')
		endian = flags & 0x01
		message.endian = endian			# now, endian is known
		message_type = CORBA.demarshal(message, 'octet')
		message_size = CORBA.demarshal(message, 'unsigned_long')
		if magic == 'GIOP' and GIOP_version.major == 1 and GIOP_version.minor == 2 and message_type == 0 :
			request_header = GIOP.RequestHeader_1_2.demarshal(message)
			interface = request_header.target._v
			if self.itf.has_key(interface) == False :
				print "unknown interface '%s'." % interface
				reply_status = GIOP.SYSTEM_EXCEPTION
				reply_body = CDR.OutputBuffer()
				CORBA.marshal(reply_body, 'string', 'IDL:CORBA/NO_IMPLEMENT:1.0')
				CORBA.marshal(reply_body, 'unsigned_long', 11)
				CORBA.marshal(reply_body, 'unsigned_long', 1)	# COMPLETED_NO
			else :
				classname = self.itf[interface]
				op = request_header.operation
				if hasattr(classname, op) == False :
					print "unknown operation '%s'." % op
					reply_status = GIOP.SYSTEM_EXCEPTION
					reply_body = CDR.OutputBuffer()
					CORBA.marshal(reply_body, 'string', 'IDL:CORBA/BAD_OPERATION:1.0')
					CORBA.marshal(reply_body, 'unsigned_long', 13)
					CORBA.marshal(reply_body, 'unsigned_long', 1)	# COMPLETED_NO
				else :
					srv_op = '_skel_' + op
					(reply_status, reply_body) = getattr(classname, srv_op)(message)
					if reply_status == None :
						return (None, message.read())		# oneway
			reply = CDR.OutputBuffer()
			GIOP.ReplyHeader_1_2(
				request_id=request_header.request_id,
				reply_status=reply_status,
				service_context=IOP.ServiceContextList([])
			).marshal(reply)
			reply.write(reply_body.getvalue())
			reply_body.close()
			buffer = CDR.OutputBuffer()
			GIOP.MessageHeader_1_1(
				magic="GIOP",
				GIOP_version=GIOP.Version(major=1, minor=2),
				flags=0x01,			# flags : little endian
				message_type=1,		# Reply
				message_size=len(reply.getvalue())
			).marshal(buffer)
			buffer.write(reply.getvalue())
			reply.close()
			s = buffer.getvalue()
			buffer.close()
			return (s, message.read())





import PyIDL as CORBA
import struct
import StringIO

class OutputBuffer(object):

	def __init__(self, buf=None):
		if buf is None :
			buf = StringIO.StringIO()
		self._buf = buf
		self._pos = 0
		self._endian = 1	# little endian

	def _get_endian(self):
		return self._endian

	endian = property(fget=_get_endian)

	def getvalue(self):
		return self._buf.getvalue()

	def close(self):
		self._buf.close()

	def write(self, str):
		self._buf.write(str)

	def _align(self, size):
		while (self._pos % size) != 0 :
			self._buf.write(chr(0))
			self._pos += 1

	def _pack(self, fmt, value):
		str = struct.pack(fmt, value)
		self._buf.write(str)
		self._pos += len(str)

	def char__marshal(self, value):
		self._pack('c', value)

	def wchar__marshal(self, value):
		raise NotImplementedError

	def octet__marshal(self, value):
		self._pack('B', value)

	def short__marshal(self, value):
		self._align(2)
		self._pack('h', value)

	def unsigned_short__marshal(self, value):
		self._align(2)
		self._pack('H', value)

	def long__marshal(self, value):
		self._align(4)
		self._pack('l', value)

	def unsigned_long__marshal(self, value):
		self._align(4)
		self._pack('L', value)

	def long_long__marshal(self, value):
		raise NotImplementedError

	def unsigned_long_long__marshal(self, value):
		raise NotImplementedError

	def float__marshal(self, value):
		self._align(4)
		self._pack('f', value)

	def double__marshal(self, value):
		self._align(8)
		self._pack('d', value)

	def long_double__marshal(self, value):
		raise NotImplementedError

	def boolean__marshal(self, value):
		if value == True :
			self._pack('B', 1)
		else :
			self._pack('B', 0)

	def string__marshal(self, value):
		self._align(4)
		value += chr(0)
		length = len(value)
		self._pack('L', length)
		self._pack(str(length) + 's', value)

	def wstring__marshal(self, value):
		raise NotImplementedError


class InputBuffer(object):

	def __init__(self, input=None, endian=None):
		if input == None :
			self._buf = StringIO.StringIO('')
		if isinstance(input, str) :
			self._buf = StringIO.StringIO(input)
 		else :
			self._buf = input
		self._pos = 0
		self._endian = endian

	def _set_endian(self, value):
		self._endian = value

	def _get_endian(self):
		return self._endian

	endian = property(fset=_set_endian, fget=_get_endian)

	def getvalue(self):
		return self._buf.getvalue()

	def close(self):
		self._buf.close()

	def read(self, size=-1):
		return self._buf.read(size)

	def _align(self, size):
		while (self._pos % size) != 0 :
			dummy = self._buf.read(1)
			self._pos += 1

	def _unpack(self, fmt):
		size = struct.calcsize(fmt)
		chunk = self._buf.read(size)
		if len(chunk) < size :
			print "Not enough data!"
			raise CORBA.SystemException("IDL:CORBA/INTERNAL:1.0", 8, CORBA_COMPLETED_MAYBE)
		self._pos += size
		return struct.unpack(fmt, chunk)[0]

	def char__demarshal(self):
		return self._unpack('c')

	def wchar__demarshal(self):
		raise NotImplementedError

	def octet__demarshal(self):
		return self._unpack('B')

	def short__demarshal(self):
		self._align(2)
		return self._unpack('h')

	def unsigned_short__demarshal(self):
		self._align(2)
		return self._unpack('H')

	def long__demarshal(self):
		self._align(4)
		return self._unpack('l')

	def unsigned_long__demarshal(self):
		self._align(4)
		return self._unpack('L')

	def long_long__demarshal(self):
		raise NotImplementedError

	def unsigned_long_long__demarshal(self):
		raise NotImplementedError

	def float__demarshal(self):
		self._align(4)
		return self._unpack('f')

	def double__demarshal(self):
		self._align(8)
		return self._unpack('d')

	def long_double__demarshal(self):
		raise NotImplementedError

	def boolean__demarshal(self):
		return self._unpack('B') == 1

	def string__demarshal(self):
		self._align(4)
		length = self._unpack('L')
		s = self._unpack(str(length) + 's')
		return s[:len(s)-1]

	def wstring__demarshal(self):
		raise NotImplementedError




from distutils.core import setup

setup(
    name="PyIDL",
    version="0.28",
    description="CORBA-IDL runtime",
    author="Francois Perrad",
    author_email="francois.perrad@gadz.org",
    license="Artistic",
    packages=['PyIDL'],
)

